{******************************************************************************}
{                                RotatebleLabel
{                                --------------
{
{                    Copyright  2002-2012 Pieter Zijlstra
{
{ Label component for which the caption can be drawn in a angle.
{
{ E-mail: p.zylstra@hccnet.nl
{ Website: http://home.hccnet.nl/p.zylstra/
{===============================================================================
{                           This software is FREEWARE
{                           -------------------------
{
{ You may freely use it in any software, including commercial software, but
{ be-aware that the code is provided as-is, with no implied warranty.
{===============================================================================
{
{ Notes:
{ - Delphi version(s) 1, 2, 3, 5, 6, 7, 2005, 2006, 2007, 2009, 2010, XE & XE2.
{ - This will only work for TrueType fonts.
{
{ Version history:
{ 1.0.0.0 29 mar 2002 - First public release.
{ 1.1.0.0 30 jun 2002 - Modified for Delphi 6
{ 1.1.1.0 26 dec 2002 - Checked for D7, no changes.
{ 1.1.2.0 03 feb 2003 - Added e-mail and website in comments.
{ 2.0.0.0 14 may 2005 - Published as much as possible of the standard properties.
{ 2.1.0.0 24 sep 2009 - Checked for D2009
{ 2.2.0.0 16 mar 2010 - Checked for D2010
{ 2.3.0.0 11 apr 2012 - Checked for DXE2
{******************************************************************************}
unit RotLabel;

interface

{$I CompVers.inc}

uses
  {$IFDEF DELPHI1}
    WinTypes, WinProcs,
  {$ELSE}
    Windows,
  {$ENDIF}
  Messages, SysUtils, Classes, Controls, Graphics;

type
  TRotation = -360..360;

  TCustomRotatebleLabel = class(TGraphicControl)
  private
    FAutoSize: Boolean;
    FRCos: Double;
    FRSin: Double;
    FRotation: TRotation;
    FTextHeight: Integer;
    FTextWidth: Integer;
    FOnMouseLeave: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;
    procedure AdjustBounds;
    procedure DoDrawText(var Rect: TRect);
    function  GetTransparent: Boolean;
    procedure SetRotation(Value: TRotation);
    procedure SetTransparent(Value: Boolean);
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    function  GetLabelText: string; virtual;
    procedure Paint; override;
{$IFDEF COMPILER6_UP}
    procedure SetAutoSize(Value: Boolean); override;
{$ELSE}
    procedure SetAutoSize(Value: Boolean); virtual;
{$ENDIF}
    property AutoSize: Boolean read FAutoSize write SetAutoSize default True;
    property Rotation: TRotation read FRotation write SetRotation;
    property Transparent: Boolean read GetTransparent write SetTransparent default False;
  public
    constructor Create(AOwner: TComponent); override;
    property Canvas;
    property ParentFont default False;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

  TRotatebleLabel = class(TCustomRotatebleLabel)
  published
    property Align;
//D1  property Alignment;
{$IFDEF COMPILER4_UP}
    property Anchors;
{$ENDIF}
    property AutoSize;
{$IFDEF COMPILER4_UP}
    property BiDiMode;
{$ENDIF}
    property Caption;
    property Color;
{$IFDEF COMPILER4_UP}
    property Constraints;
{$ENDIF}
    property DragCursor;
{$IFDEF COMPILER4_UP}
    property DragKind;
{$ENDIF}
    property DragMode;
    property Enabled;
//D1  property FocusControl;
    property Font;
{$IFDEF COMPILER4_UP}
    property ParentBiDiMode;
{$ENDIF}
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Rotation;
//D1  property ShowAccelChar;
    property ShowHint;
    property Transparent;
{$IFDEF COMPILER3_UP}
//D3    property Layout;
{$ENDIF}
    property Visible;
//D1  property WordWrap;
    property OnClick;
{$IFDEF COMPILER5_UP}
    property OnContextPopup;
{$ENDIF}
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
{$IFDEF COMPILER4_UP}
    property OnEndDock;
{$ENDIF}
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
{$IFDEF COMPILER4_UP}
    property OnStartDock;
{$ENDIF}
{$IFDEF COMPILER2_UP}
    property OnStartDrag;
{$ENDIF}
  end;

procedure Register;

implementation

constructor TCustomRotatebleLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  FAutoSize    := True;

  Font.Name  := 'Arial';
  Font.Style := [fsBold];
  Font.Size  := 12;

  FRotation := 90;
  FRCos     :=  0;
  FRSin     :=  1;
end;

function TCustomRotatebleLabel.GetLabelText: String;
begin
  Result := Caption;
end;

procedure TCustomRotatebleLabel.DoDrawText(var Rect: TRect);
var
  lf: TLogFont;
  tf: TFont;
  Text: string;
  X, Y: Integer;
begin
  Text := GetLabelText;
  Canvas.Font := Font;

  with Canvas do
  begin
    tf := TFont.Create;
    tf.Assign(Font);
    GetObject(tf.Handle, sizeof(lf), @lf);
    lf.lfEscapement  := 10 * FRotation;  {angle *10}
    lf.lfOrientation := 10 * FRotation;
    lf.lfOutPrecision := OUT_TT_ONLY_PRECIS; // Only supported by True Type fonts.
    tf.Handle := CreateFontIndirect(lf);
    Font.Assign(tf);
    tf.Free;

    { Calculate X, Y so that the caption is always centered in the box
      TextOut always start drawing at given point to any direction     }
    X := Round(0.5 * (Width  - FTextWidth * FRCos - FTextHeight * FRSin));
    Y := Round(0.5 * (Height + FTextWidth * FRSin - FTextHeight * FRCos));

    if not Enabled then
    begin
      Font.Color := clBtnHighlight;
      TextOut(X+1, Y+1, Caption);
      Font.Color := clBtnShadow;
      TextOut(X, Y, Caption);
    end
    else
      TextOut(X, Y, Text);
  end;
end;

procedure TCustomRotatebleLabel.Paint;
var
  Rect: TRect;
begin
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    DoDrawText(Rect);
  end;
end;

procedure TCustomRotatebleLabel.AdjustBounds;
var
  DC: HDC;
  Tw, Th: Integer;
begin
  DC := GetDC(0);
  try
    Canvas.Handle := DC;
    Canvas.Font   := Font;
    FTextWidth    := Canvas.TextWidth(Text);
    FTextHeight   := Canvas.TextHeight(Text);
    Canvas.Handle := 0;
  finally
    ReleaseDC(0, DC);
  end;
  if not (csReading in ComponentState) and FAutoSize then
  begin
    { Calculate new rectangle coordinates }
    Tw := Round(Abs(FTextWidth * FRCos) +  Abs(FTextHeight * FRSin));
    Th := Round(Abs(FTextWidth * FRSin) +  Abs(FTextHeight * FRCos));
    SetBounds(Left + (Width-Tw) div 2, Top + (Height-Th) div 2, Tw, Th);
  end;
end;

procedure TCustomRotatebleLabel.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize := Value;
    AdjustBounds;
  end;
end;

procedure TCustomRotatebleLabel.SetRotation(Value: TRotation);
begin
  if FRotation <> Value then
  begin
    FRotation := Value;
    FRCos     := Cos(FRotation * PI / 180);
    FRSin     := Sin(FRotation * PI / 180);
    Invalidate;
    AdjustBounds;
  end;
end;

function TCustomRotatebleLabel.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

procedure TCustomRotatebleLabel.SetTransparent(Value: Boolean);
begin
  if Transparent <> Value then
  begin
    if Value then
      ControlStyle := ControlStyle - [csOpaque]
    else
      ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
end;

procedure TCustomRotatebleLabel.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
  AdjustBounds;
end;

procedure TCustomRotatebleLabel.CMFontChanged(var Message: TMessage);
begin
  inherited;
  AdjustBounds;
end;

procedure TCustomRotatebleLabel.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseEnter) then
    FOnMouseEnter(Self);
end;

procedure TCustomRotatebleLabel.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseLeave) then
    FOnMouseLeave(Self);
end;

procedure Register;
begin
  RegisterComponents('PZGoodies', [TRotatebleLabel]);
end;

end.
